/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kbounce-26.04.0/src/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KBounceMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KBounceMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN17KBounceMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KBounceMainWindow",
        "newGame",
        "",
        "pauseGame",
        "closeGame",
        "gameOverNow",
        "configureSettings",
        "readSettings",
        "settingsChanged",
        "setSounds",
        "val",
        "showHighscore",
        "displayLevel",
        "level",
        "displayScore",
        "score",
        "displayFilled",
        "filled",
        "displayLives",
        "lives",
        "displayTime",
        "time",
        "gameStateChanged",
        "KBounceGameWidget::State",
        "state"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'newGame'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'pauseGame'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'closeGame'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'gameOverNow'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'configureSettings'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'readSettings'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'settingsChanged'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setSounds'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'showHighscore'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'displayLevel'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'displayScore'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'displayFilled'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'displayLives'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'displayTime'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'gameStateChanged'
        QtMocHelpers::SlotData<void(KBounceGameWidget::State)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KBounceMainWindow, qt_meta_tag_ZN17KBounceMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KBounceMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KBounceMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KBounceMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KBounceMainWindowE_t>.metaTypes,
    nullptr
} };

void KBounceMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KBounceMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newGame(); break;
        case 1: _t->pauseGame(); break;
        case 2: _t->closeGame(); break;
        case 3: _t->gameOverNow(); break;
        case 4: _t->configureSettings(); break;
        case 5: _t->readSettings(); break;
        case 6: _t->settingsChanged(); break;
        case 7: _t->setSounds((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->showHighscore(); break;
        case 9: _t->displayLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->displayScore((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->displayFilled((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->displayLives((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->displayTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->gameStateChanged((*reinterpret_cast<std::add_pointer_t<KBounceGameWidget::State>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KBounceMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KBounceMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KBounceMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KBounceMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
