// This file is generated by kconfig_compiler_kf6 from kbounce.kcfg.
// All changes you do to this file will be lost.
#ifndef KBOUNCESETTINGS_H
#define KBOUNCESETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class KBounceSettings : public KConfigSkeleton
{
  public:

    static KBounceSettings *self();
    ~KBounceSettings() override;

    /**
      Set UseRandomBackgroundPictures
    */
    static
    void setUseRandomBackgroundPictures( bool v )
    {
      if (!self()->isUseRandomBackgroundPicturesImmutable())
        self()->mUseRandomBackgroundPictures = v;
    }

    /**
      Get UseRandomBackgroundPictures
    */
    static
    bool useRandomBackgroundPictures()
    {
      return self()->mUseRandomBackgroundPictures;
    }

    /**
      Is UseRandomBackgroundPictures Immutable
    */
    static
    bool isUseRandomBackgroundPicturesImmutable()
    {
      return self()->isImmutable( QStringLiteral( "UseRandomBackgroundPictures" ) );
    }

    /**
      Set BackgroundPicturePath
    */
    static
    void setBackgroundPicturePath( const QString & v )
    {
      if (!self()->isBackgroundPicturePathImmutable())
        self()->mBackgroundPicturePath = v;
    }

    /**
      Get BackgroundPicturePath
    */
    static
    QString backgroundPicturePath()
    {
      return self()->mBackgroundPicturePath;
    }

    /**
      Is BackgroundPicturePath Immutable
    */
    static
    bool isBackgroundPicturePathImmutable()
    {
      return self()->isImmutable( QStringLiteral( "BackgroundPicturePath" ) );
    }

    /**
      Set Whether game sounds are played.
    */
    static
    void setPlaySounds( bool v )
    {
      if (!self()->isPlaySoundsImmutable())
        self()->mPlaySounds = v;
    }

    /**
      Get Whether game sounds are played.
    */
    static
    bool playSounds()
    {
      return self()->mPlaySounds;
    }

    /**
      Is Whether game sounds are played. Immutable
    */
    static
    bool isPlaySoundsImmutable()
    {
      return self()->isImmutable( QStringLiteral( "PlaySounds" ) );
    }

  protected:
    KBounceSettings();
    friend class KBounceSettingsHelper;


    // General
    bool mUseRandomBackgroundPictures;
    QString mBackgroundPicturePath;

    // Sound
    bool mPlaySounds;
};

#endif

