/********************************************************************************
** Form generated from reading UI file 'backgroundselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BACKGROUNDSELECTOR_H
#define UI_BACKGROUNDSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KBounceBackgroundSelector
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *kcfg_UseRandomBackgroundPictures;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout;
    QLabel *backgoundPicturePathLabel;
    KUrlRequester *kurlrequester;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QLabel *backgroundPicturePreview;
    QSpacerItem *verticalSpacer;
    KLineEdit *kcfg_BackgroundPicturePath;

    void setupUi(QWidget *KBounceBackgroundSelector)
    {
        if (KBounceBackgroundSelector->objectName().isEmpty())
            KBounceBackgroundSelector->setObjectName("KBounceBackgroundSelector");
        KBounceBackgroundSelector->resize(600, 270);
        gridLayout = new QGridLayout(KBounceBackgroundSelector);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        kcfg_UseRandomBackgroundPictures = new QCheckBox(KBounceBackgroundSelector);
        kcfg_UseRandomBackgroundPictures->setObjectName("kcfg_UseRandomBackgroundPictures");

        horizontalLayout_2->addWidget(kcfg_UseRandomBackgroundPictures);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        backgoundPicturePathLabel = new QLabel(KBounceBackgroundSelector);
        backgoundPicturePathLabel->setObjectName("backgoundPicturePathLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(backgoundPicturePathLabel->sizePolicy().hasHeightForWidth());
        backgoundPicturePathLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(backgoundPicturePathLabel);

        kurlrequester = new KUrlRequester(KBounceBackgroundSelector);
        kurlrequester->setObjectName("kurlrequester");
        sizePolicy.setHeightForWidth(kurlrequester->sizePolicy().hasHeightForWidth());
        kurlrequester->setSizePolicy(sizePolicy);
        kurlrequester->setMode(KFile::Directory|KFile::ExistingOnly|KFile::LocalOnly);

        horizontalLayout->addWidget(kurlrequester);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label = new QLabel(KBounceBackgroundSelector);
        label->setObjectName("label");

        horizontalLayout_3->addWidget(label);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        backgroundPicturePreview = new QLabel(KBounceBackgroundSelector);
        backgroundPicturePreview->setObjectName("backgroundPicturePreview");
        backgroundPicturePreview->setMinimumSize(QSize(160, 120));
        backgroundPicturePreview->setAlignment(Qt::AlignCenter);

        horizontalLayout_3->addWidget(backgroundPicturePreview);


        verticalLayout->addLayout(horizontalLayout_3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        kcfg_BackgroundPicturePath = new KLineEdit(KBounceBackgroundSelector);
        kcfg_BackgroundPicturePath->setObjectName("kcfg_BackgroundPicturePath");

        verticalLayout->addWidget(kcfg_BackgroundPicturePath);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(KBounceBackgroundSelector);

        QMetaObject::connectSlotsByName(KBounceBackgroundSelector);
    } // setupUi

    void retranslateUi(QWidget *KBounceBackgroundSelector)
    {
        KBounceBackgroundSelector->setWindowTitle(tr2i18n("kbouncebackgroundselector", nullptr));
        kcfg_UseRandomBackgroundPictures->setText(tr2i18n("Enable random pictures as background", nullptr));
        backgoundPicturePathLabel->setText(tr2i18n("Choose picture path:", nullptr));
        label->setText(tr2i18n("Preview", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KBounceBackgroundSelector: public Ui_KBounceBackgroundSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BACKGROUNDSELECTOR_H

